/*
This file is part of CASToR.

    CASToR is free software: you can redistribute it and/or modify it under the
    terms of the GNU General Public License as published by the Free Software
    Foundation, either version 3 of the License, or (at your option) any later
    version.

    CASToR is distributed in the hope that it will be useful, but WITHOUT ANY
    WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
    details.

    You should have received a copy of the GNU General Public License along with
    CASToR (in file GNU_GPL.TXT). If not, see <http://www.gnu.org/licenses/>.

Copyright 2017 all CASToR contributors listed below:

    --> current contributors: Thibaut MERLIN, Simon STUTE, Didier BENOIT
    --> past contributors: Valentin VIELZEUF

This is CASToR version 1.0.
*/

/* Specifications:

This class is a child of vOptimizer and must implement few pure virtual functions:
 - ReadAndCheckConfigurationFile(): read specific options from a configuration file
 - ReadAndCheckOptionsList(): read specific options from a string
 - ShowHelp(): print helps about the specific options of the projector and how it is working
 - Initialize(): initialize specific stuff of the projector

This class implements the MLEM algorithm.

*/

#ifndef IOPTIMIZERTEST_HH
#define IOPTIMIZERTEST_HH 1

#include "gVariables.hh"
#include "sAddonManager.hh"
#include "vOptimizer.hh"

class iOptimizerTest : public vOptimizer
{
  // -----------------------------------------------------------------------------------------
  // Constructor & Destructor
  public:
    iOptimizerTest();
    ~iOptimizerTest();

  // -----------------------------------------------------------------------------------------
  // Public member functions
  public:
    // Function for automatic insertion (put the class name as the parameters and do not add semi-column at the end of the line)
    FUNCTION_OPTIMIZER(iOptimizerTest)
    // Implementations of pure virtual methods for options reading
    int ReadConfigurationFile(const string& a_configurationFile);
    int ReadOptionsList(const string& a_optionsList);

  // -----------------------------------------------------------------------------------------
  // Private member functions (pure virtual in vOptimizer)
  private:
    // Implementation of pure virtual method for showing help
    void ShowHelpSpecific();
    // Implementation of pure virtual method for checking specific parameters of the optimizer
    int CheckSpecificParameters();
    // Implementation of pure virtual method for initializing specific stuff of the optimizer
    int InitializeSpecific();
    // Method for performing sensitivity operations of the optimizer
    int SensitivitySpecificOperations( FLTNB a_data, FLTNB a_forwardModel, FLTNB* ap_weight,
                                       FLTNB a_multiplicativeCorrections, FLTNB a_additiveCorrections,
                                       FLTNB a_quantificationFactor, oProjectionLine* ap_Line );
    // Method for performing data-space operations of the optimizer
    int DataSpaceSpecificOperations( FLTNB a_data, FLTNB a_forwardModel, FLTNB* a_backwardValues,
                                     FLTNB a_multiplicativeCorrections, FLTNB a_additiveCorrections,
                                     FLTNB a_quantificationFactor, oProjectionLine* ap_Line );
    // Method for performing image-space operations of the optimizer
    int ImageSpaceSpecificOperations( FLTNB a_currentImageValue, FLTNB* ap_newImageValue,
                                      FLTNB a_sensitivity, FLTNB* ap_correctionValues );

  // -----------------------------------------------------------------------------------------
  // Data members
  private:
    // Buffer float
    double m_buffer1;
    double m_buffer2;
    // Counter
    long int m_counter;
};

// Class for automatic insertion (set here the visible optimizer's name, put the class name as the parameters and do not add semi-column at the end of the line)
CLASS_OPTIMIZER(test,iOptimizerTest)

#endif

